@@ -8,6 +8,7 @@ from datetime import datetime
 from timeit import default_timer as timer
 
 import six
+from bs4 import BeautifulSoup
 from kitchen.text.display import wrap
 
 from . import exceptions
@@ -148,6 +149,7 @@ class Content(object):
             data['type'] = 'Comment'
             data['level'] = comment.nested_level
             data['body'] = comment.body
+            data['html'] = comment.body_html
             data['created'] = cls.humanize_timestamp(comment.created_utc)
             data['score'] = '{0} pts'.format(
                 '-' if comment.score_hidden else comment.score)
@@ -155,11 +157,16 @@ class Content(object):
             data['is_author'] = (name == sub_name)
             data['flair'] = flair
             data['likes'] = comment.likes
-            data['gold'] = comment.gilded > 0
+            data['gold'] = comment.gilded
             data['permalink'] = permalink
             data['stickied'] = stickied
             data['hidden'] = False
             data['saved'] = comment.saved
+            if comment.edited:
+                data['edited'] = '(edit {})'.format(
+                        cls.humanize_timestamp(comment.edited))
+            else:
+                data['edited'] = ''
         else:
             # Saved comments don't have a nested level and are missing a couple
             # of fields like ``submission``. As a result, we can only load a
@@ -185,10 +192,15 @@ class Content(object):
             data['created'] = cls.humanize_timestamp(comment.created_utc)
             data['saved'] = comment.saved
             data['stickied'] = stickied
-            data['gold'] = comment.gilded > 0
+            data['gold'] = comment.gilded
             data['author'] = author
             data['flair'] = flair
             data['hidden'] = False
+            if comment.edited:
+                data['edited'] = '(edit {})'.format(
+                        cls.humanize_timestamp(comment.edited))
+            else:
+                data['edited'] = ''
 
         return data
 
@@ -217,6 +229,7 @@ class Content(object):
         data['type'] = 'Submission'
         data['title'] = sub.title
         data['text'] = sub.selftext
+        data['html'] = sub.selftext_html or ''
         data['created'] = cls.humanize_timestamp(sub.created_utc)
         data['created_long'] = cls.humanize_timestamp(sub.created_utc, True)
         data['comments'] = '{0} comments'.format(sub.num_comments)
@@ -227,13 +240,21 @@ class Content(object):
         data['flair'] = '[{0}]'.format(flair.strip(' []')) if flair else ''
         data['url_full'] = sub.url
         data['likes'] = sub.likes
-        data['gold'] = sub.gilded > 0
+        data['gold'] = sub.gilded
         data['nsfw'] = sub.over_18
         data['stickied'] = sub.stickied
         data['hidden'] = sub.hidden
         data['xpost_subreddit'] = None
         data['index'] = None  # This is filled in later by the method caller
         data['saved'] = sub.saved
+        if sub.edited:
+            data['edited'] = '(edit {})'.format(
+                    cls.humanize_timestamp(sub.edited))
+            data['edited_long'] = '(edit {})'.format(
+                    cls.humanize_timestamp(sub.edited, True))
+        else:
+            data['edited'] = ''
+            data['edited_long'] = ''
 
         if sub.url.split('/r/')[-1] == sub.permalink.split('/r/')[-1]:
             data['url'] = 'self.{0}'.format(data['subreddit'])
@@ -315,6 +336,22 @@ class Content(object):
             out.extend(lines)
         return out
 
+    @staticmethod
+    def extract_links(html):
+        """
+        Extract a list of hyperlinks from an HTML document.
+        """
+        links = []
+        soup = BeautifulSoup(html, 'html.parser')
+        for link in soup.findAll('a'):
+            href = link.get('href')
+            if not href:
+                continue
+            if href.startswith('/'):
+                href = 'https://www.reddit.com' + href
+            links.append({'text': link.text, 'href': href})
+        return links
+
 
 class SubmissionContent(Content):
     """
@@ -510,12 +547,41 @@ class SubredditContent(Content):
         if resource_root == 'user':
             resource_root = 'u'
         elif resource_root.startswith('user/'):
+            # Special check for multi-reddit resource roots
+            # E.g.
+            #     before: resource_root = "user/civilization_phaze_3/m"
+            #     After:  resource_root = "u/civilization_phaze_3/m"
             resource_root = 'u' + resource_root[4:]
 
-        # There should at most two parts left, the resource and the order
+        # The parts left should be in one of the following forms:
+        #    [resource]
+        #    [resource, order]
+        #    [resource, user_room, order]
+
+        user_rooms = ['overview', 'submitted', 'comments']
+        private_user_rooms = ['upvoted', 'downvoted', 'hidden', 'saved']
+        user_room = None
+
         if len(parts) == 1:
+            # E.g. /r/python
+            #    parts = ["python"]
+            #    resource = "python"
+            #    resource_order = None
             resource, resource_order = parts[0], None
+        elif resource_root == 'u' and len(parts) in [2, 3] \
+                and parts[1] in user_rooms + private_user_rooms:
+            # E.g. /u/spez/submitted/top ->
+            #    parts = ["spez", "submitted", "top"]
+            #    resource = "spez"
+            #    user_room = "submitted"
+            #    resource_order = "top"
+            resource, user_room = parts[:2]
+            resource_order = parts[2] if len(parts) == 3 else None
         elif len(parts) == 2:
+            # E.g. /r/python/top
+            #    parts = ["python", "top"]
+            #    resource = "python
+            #    resource_order = "top"
             resource, resource_order = parts
         else:
             raise InvalidSubreddit('`{}` is an invalid format'.format(name))
@@ -531,6 +597,8 @@ class SubredditContent(Content):
 
         display_order = order
         display_name = '/'.join(['', resource_root, resource])
+        if user_room and resource_root == 'u':
+            display_name += '/' + user_room
 
         # Split the order from the period E.g. controversial-all, top-hour
         if order and '-' in order:
@@ -603,22 +671,23 @@ class SubredditContent(Content):
             if not reddit.is_oauth_session():
                 raise exceptions.AccountError('Not logged in')
             else:
+                user_room = user_room or 'overview'
                 order = order or 'new'
-                submissions = reddit.user.get_overview(sort=order, limit=None)
-
-        elif resource_root == 'u' and resource == 'saved':
-            if not reddit.is_oauth_session():
-                raise exceptions.AccountError('Not logged in')
-            else:
-                order = order or 'new'
-                submissions = reddit.user.get_saved(sort=order, limit=None)
+                period = period or 'all'
+                method = getattr(reddit.user, 'get_%s' % user_room)
+                submissions = method(sort=order, time=period, limit=None)
 
         elif resource_root == 'u':
+            user_room = user_room or 'overview'
+            if user_room not in user_rooms:
+                # Tried to access a private room like "u/me/hidden" for a
+                # different redditor
+                raise InvalidSubreddit('Unavailable Resource')
             order = order or 'new'
             period = period or 'all'
             redditor = reddit.get_redditor(resource)
-            submissions = redditor.get_overview(
-                sort=order, time=period, limit=None)
+            method = getattr(redditor, 'get_%s' % user_room)
+            submissions = method(sort=order, time=period, limit=None)
 
         elif resource == 'front':
             if order in (None, 'hot'):
